//
//  InformationViewController.m
//  P52NDemo
//
//  Created by mac on 2020/11/20.
//

#import "InformationViewController.h"
#import "InformationCell.h"
#import <P52NSDK/P52NSDK.h>
#import <SVProgressHUD/SVProgressHUD.h>

@interface InformationViewController ()<UITableViewDelegate, UITableViewDataSource, AsP52NDelegate>
@property (strong, nonatomic) IBOutlet UITableView *tableView;
@end

@implementation InformationViewController{
    NSArray *dataArray;
    NSMutableArray *valueArray;
    AsP52N *asP52N;
    NSString *_firmwareVersion;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    _firmwareVersion = @"";
    dataArray = [NSArray arrayWithObjects:@"Manufacture number:", @"App version:", @"Application compilation:", @"Baseband version:", @"Baseband compilation:", @"System version:", @"Power-on time:", nil];
    valueArray = [[NSMutableArray alloc] init];
    self.tableView.tableFooterView = [[UIView alloc] initWithFrame:CGRectZero];
    asP52N = [AsP52N sharedInstance];
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Information"];
    [asP52N getFirmwareVersion];
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
}
#pragma mark - Table view data source
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return valueArray.count;
}
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    NSString *cellIdentifier = @"InformationCell";
    InformationCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if(!cell) {
        cell = [[[NSBundle mainBundle] loadNibNamed:@"InformationCell" owner:self options:nil] firstObject];
    }
    cell.nameLabel.text = [dataArray objectAtIndex:indexPath.row];
    cell.valueLabel.text = [valueArray objectAtIndex:indexPath.row];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.accessoryType = UITableViewCellAccessoryNone;
    return cell;
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N receivedFirmwareVersion:(NSString *)firmwareVersion executionStatus:(AsP52NExecutionStatus)statusCode {
    _firmwareVersion = firmwareVersion?:@"";
    [asP52N getReaderInfo];
}
- (void)AsP52N:(AsP52N *)AsP52N receivedReaderInfo:(NSDictionary *)readerInfo {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    if ([readerInfo allValues].count <= 0) {
        return;
    }
    NSString *deviceSerialNumber = [readerInfo valueForKey:@"deviceSerialNumber"]?:@"";
    NSString *appVersion = [readerInfo valueForKey:@"appVersion"]?:@"";
    NSString *applicationCompilation = [readerInfo valueForKey:@"applicationCompilation"]?:@"";
    NSString *basebandCompilation = [readerInfo valueForKey:@"basebandCompilation"]?:@"";
    NSString *systemVersion = [readerInfo valueForKey:@"systemVersion"]?:@"";
    NSString *powerOnTime = [readerInfo valueForKey:@"powerOnTime"]?:@"";
    [valueArray addObject:deviceSerialNumber];
    [valueArray addObject:appVersion];
    [valueArray addObject:applicationCompilation];
    [valueArray addObject:_firmwareVersion];
    [valueArray addObject:basebandCompilation];
    [valueArray addObject:systemVersion];
    [valueArray addObject:powerOnTime];
    [self.tableView reloadData];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state != AsP52NetworkStateConnected) {
            [self.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}
#pragma mark - Private
- (void)dismissHUD {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [SVProgressHUD dismiss];
    }];
}
@end
