//
//  ReaderTimeViewController.m
//  P52NDemo
//
//  Created by mac on 2020/11/30.
//

#import "ReaderTimeViewController.h"
#import <P52NSDK/P52NSDK.h>
#import <SVProgressHUD/SVProgressHUD.h>
#import "Utils.h"

@interface ReaderTimeViewController ()<AsP52NDelegate, UITextFieldDelegate>
@property (strong, nonatomic) IBOutlet UITextField *textField;
@end

@implementation ReaderTimeViewController
{
    AsP52N *asP52N;
}
- (void)viewDidLoad {
    [super viewDidLoad];
    asP52N = [AsP52N sharedInstance];
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Reader time"];
    [asP52N getReaderTime];
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N receivedReaderTime:(NSString *)readerTime {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    self.textField.text = readerTime;
}
- (void)AsP52N:(AsP52N *)AsP52N receivedSetReaderTimeStatus:(AsP52NExecutionStatus)statusCode {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    NSString *failedString = [asP52N getMessageWithResultCode:statusCode];
    [Utils showAlert:@"" message:failedString];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state != AsP52NetworkStateConnected) {
            [self.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}
#pragma mark - UITextFieldDelegate
- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}
#pragma mark - Action
- (IBAction)onClickDoneBarItem:(UIBarButtonItem *)sender {
    [self.view endEditing:YES];
    if (self.textField.text.length <= 0) {
        return;
    }
    long time = [self totalSecondsFormatted:self.textField.text];
    if (time == 0) {
        return;
    }
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Set reader time"];
    [asP52N setReaderTime:time];
}
#pragma mark - System
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
}
#pragma mark - Private
- (void)dismissHUD {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [SVProgressHUD dismiss];
    }];
}
- (long)totalSecondsFormatted:(NSString *)time{
    NSDateFormatter *format = [[NSDateFormatter alloc] init];
    format.timeZone = [NSTimeZone systemTimeZone];
    format.dateFormat = @"yyyy.MM.dd HH:mm:ss";
    NSDate *date = [format dateFromString:time];
    if (date == nil) {
        format.dateFormat = @"yyyy.MM.dd";
        date = [format dateFromString:time];
        if (date == nil) {
            format.dateFormat = @"yyyy.MM.dd HH:mm:ss";
            date = [format dateFromString:[NSString stringWithFormat:@"%@ %@", [self getCurrentDate], time]];
        }
    }
    return date.timeIntervalSince1970;
}
- (NSString *)getCurrentDate {
    NSDate *date = [NSDate date];
    NSDateFormatter *format = [[NSDateFormatter alloc] init];
    format.timeZone = [NSTimeZone systemTimeZone];
    format.dateFormat = @"yyyy.MM.dd";
    NSString *today = [format stringFromDate:date];
    return today;
}
@end
