//
//  AutoDormancyViewController.m
//  P52NDemo
//
//  Created by mac on 2020/11/26.
//

#import "AutoDormancyViewController.h"
#import <P52NSDK/P52NSDK.h>
#import <SVProgressHUD/SVProgressHUD.h>
#import "Utils.h"
@interface AutoDormancyViewController ()<UITextFieldDelegate, AsP52NDelegate>

@property (strong, nonatomic) IBOutlet UISwitch *onOffSwitch;
@property (strong, nonatomic) IBOutlet UITextField *timeField;
@end

@implementation AutoDormancyViewController{
    AsP52N *asP52N;
}
- (void)viewDidLoad {
    [super viewDidLoad];
    asP52N = [AsP52N sharedInstance];
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Auto dormancy"];
    [asP52N getReaderAutoDormancy];
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
}
#pragma mark - Action
- (IBAction)onClickOnOffSwitch:(UISwitch *)sender {
    if (sender.isOn) {

    } else {

    }
}
- (IBAction)onClickDoneBarItem:(UIBarButtonItem *)sender {
    [self.view endEditing:YES];
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Set auto dormancy"];
    [asP52N setReaderAutoDormancy:self.onOffSwitch.isOn time:[self.timeField.text intValue]];
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N receivedReaderAutoDormancy:(BOOL)autoDormancy dormancyTime:(int)dormancyTime {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    [self.onOffSwitch setOn:autoDormancy];
    self.timeField.text = [NSString stringWithFormat:@"%d", dormancyTime];
}
- (void)AsP52N:(AsP52N *)AsP52N receivedSetAutoDormancyStatus:(AsP52NExecutionStatus)statusCode {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    NSString *failedString = [asP52N getMessageWithResultCode:statusCode];
    [Utils showAlert:@"" message:failedString];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state != AsP52NetworkStateConnected) {
            [self.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}
#pragma mark - UITextFieldDelegate
- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    if ([[textField.text stringByAppendingString:string] intValue] < 0 || [[textField.text stringByAppendingString:string] intValue] > 65535) {
        return NO;
    } else {
        return YES;
    }
}
#pragma mark - System
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
}
#pragma mark - Private
- (void)dismissHUD {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [SVProgressHUD dismiss];
    }];
}
@end
