//
//  BandTypeViewController.m
//  P52NDemo
//
//  Created by mac on 2020/11/26.
//

#import "BandTypeViewController.h"
#import <SVProgressHUD/SVProgressHUD.h>
#import "Utils.h"

@interface BandTypeViewController ()<AsP52NDelegate, UITableViewDelegate, UITableViewDataSource>
@property (strong, nonatomic) IBOutlet UITableView *tableView;
@end

@implementation BandTypeViewController{
    NSArray *dataArray;
    AsP52N *asP52N;
    NSString *type;
}
- (void)viewDidLoad {
    [super viewDidLoad];
    switch (_basebandType) {
        case BasebandTypeRate:{
            self.navigationItem.title = @"Rate";
            type = @"Rate";
            dataArray = [NSArray arrayWithObjects:@"0", @"1", @"2", @"3", @"4", @"Auto", nil];
        }
            break;
        case BasebandTypeQValue:{
            self.navigationItem.title = @"QValue";
            type = @"QValue";
            dataArray = [NSArray arrayWithObjects:@"0", @"1", @"2", @"3", @"4", @"5", @"6", @"7", @"8", @"9", @"10", @"11", @"12", @"13", @"14", @"15", nil];
        }
            break;
        case BasebandTypeSession:{
            self.navigationItem.title = @"Session";
            type = @"Session";
            dataArray = [NSArray arrayWithObjects:@"Session0", @"Session1", @"Session2", @"Session3", nil];
        }
            break;
        case BasebandTypeInventoryFlag:{
            self.navigationItem.title = @"Inventory flag";
            type = @"Inventory flag";
            dataArray = [NSArray arrayWithObjects:@"InventoryFlag_A", @"InventoryFlag_B", @"InventoryFlag_AB", nil];
        }
            break;
        default:
            break;
    }
    self.tableView.tableFooterView = [[UIView alloc] initWithFrame:CGRectZero];
    asP52N = [AsP52N sharedInstance];
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
}
#pragma mark - Action
- (IBAction)onClickDoneBarItem:(UIBarButtonItem *)sender {
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:[NSString stringWithFormat:@"Set %@", type]];
    switch (_basebandType) {
        case BasebandTypeRate:{
            [asP52N setReaderBasebandRate:(AsP52NBasebandRateStatus)_selectedType QValue:_qValue session:_session inventoryFlag:_inventoryFlag];
        }
            break;
        case BasebandTypeQValue:{
            [asP52N setReaderBasebandRate:_rateStatus QValue:(int)_selectedType session:_session inventoryFlag:_inventoryFlag];
        }
            break;
        case BasebandTypeSession:{
            [asP52N setReaderBasebandRate:_rateStatus QValue:_qValue session:(AsP52NSessionType)_selectedType inventoryFlag:_inventoryFlag];
        }
            break;
        case BasebandTypeInventoryFlag:{
            [asP52N setReaderBasebandRate:_rateStatus QValue:_qValue session:_session inventoryFlag:(AsP52NInventoryFlag)_selectedType];
        }
            break;
        default:
            break;
    }
}
#pragma mark - Table view data source
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return dataArray.count;
}
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    NSString *cellIdentifier = @"tableViewCell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if(!cell){
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellIdentifier];
    }
    cell.textLabel.text = [dataArray objectAtIndex:indexPath.row];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    if (_selectedType == indexPath.row) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    } else {
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    return cell;
}
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    _selectedType = indexPath.row;
    [self.tableView reloadData];
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N
receivedSetBasebandStatus:(AsP52NExecutionStatus)statusCode {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    NSString *failedString = [asP52N getMessageWithResultCode:statusCode];
    [Utils showAlert:@"" message:failedString];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state != AsP52NetworkStateConnected) {
            [self.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}
#pragma mark - Private
- (void)dismissHUD {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [SVProgressHUD dismiss];
    }];
}
@end
