//
//  BaseBandViewController.m
//  P52NDemo
//
//  Created by mac on 2020/11/26.
//

#import "BaseBandViewController.h"
#import "BandTypeViewController.h"
#import <P52NSDK/P52NSDK.h>
#import <SVProgressHUD/SVProgressHUD.h>

@interface BaseBandViewController ()<AsP52NDelegate>
@property (strong, nonatomic) IBOutlet UILabel *rateLabel;
@property (strong, nonatomic) IBOutlet UILabel *qValueLabel;
@property (strong, nonatomic) IBOutlet UILabel *sessionLabel;
@property (strong, nonatomic) IBOutlet UILabel *inventoryFlagLabel;
@end

@implementation BaseBandViewController{
    AsP52N *asP52N;
    NSInteger selectedType;
    AsP52NBasebandRateStatus rateStatus;
    int qValueStatus;
    AsP52NSessionType sessionType;
    AsP52NInventoryFlag inventoryFlagStatus;
}
- (void)viewDidLoad {
    [super viewDidLoad];
    self.tableView.tableFooterView = [[UIView alloc] initWithFrame:CGRectZero];
    asP52N = [AsP52N sharedInstance];
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Base band"];
    [asP52N getReaderBasebandInformation];
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N receivedReaderBasebandRate:(AsP52NBasebandRateStatus)basebandRate QValue:(int)qValue session:(AsP52NSessionType)session inventoryFlag:(AsP52NInventoryFlag)inventoryFlag {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    rateStatus = basebandRate;
    qValueStatus = qValue;
    sessionType = session;
    inventoryFlagStatus = inventoryFlag;
    [self getBasebandRate:basebandRate];
    [self getBasebandQValue:qValue];
    [self getBasebandSession:session];
    [self getBasebandInventoryFlag:inventoryFlag];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state != AsP52NetworkStateConnected) {
            [self.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}
#pragma mark - Navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    BandTypeViewController *bandTypeVC = segue.destinationViewController;
    bandTypeVC.rateStatus = rateStatus;
    bandTypeVC.qValue = qValueStatus;
    bandTypeVC.session = sessionType;
    bandTypeVC.inventoryFlag = inventoryFlagStatus;
    if ([segue.identifier isEqualToString:@"RateID"]) {
        bandTypeVC.basebandType = BasebandTypeRate;
        bandTypeVC.selectedType = rateStatus;
    } else if ([segue.identifier isEqualToString:@"QValueID"]) {
        bandTypeVC.basebandType = BasebandTypeQValue;
        bandTypeVC.selectedType = qValueStatus;
    } else if ([segue.identifier isEqualToString:@"SessionID"]) {
        bandTypeVC.basebandType = BasebandTypeSession;
        bandTypeVC.selectedType = sessionType;
    } else if ([segue.identifier isEqualToString:@"InventoryFlagID"]) {
        bandTypeVC.basebandType = BasebandTypeInventoryFlag;
        bandTypeVC.selectedType = inventoryFlagStatus;
    }
}
#pragma mark - Private
- (void)dismissHUD {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [SVProgressHUD dismiss];
    }];
}
- (void)getBasebandRate:(AsP52NBasebandRateStatus)rate {
    NSString *rateString = @"";
    switch (rate) {
        case AsP52NBasebandRateStatus40KHz:{
            rateString = @"0";
            selectedType = 0;
        }
            break;
        case AsP52NBasebandRateStatus250KHz:{
            rateString = @"1";
            selectedType = 1;
        }
            break;
        case AsP52NBasebandRateStatus300KHz:{
            rateString = @"2";
            selectedType = 2;
        }
            break;
        case AsP52NBasebandRateStatus400KHz:{
            rateString = @"3";
            selectedType = 3;
        }
            break;
        case AsP52NBasebandRateStatus320KHz:{
            rateString = @"4";
            selectedType = 4;
        }
            break;
        case AsP52NBasebandRateStatusAuto:{
            rateString = @"Auto";
            selectedType = 5;
        }
            break;
        default:
            break;
    }
    self.rateLabel.text = rateString;
}
- (void)getBasebandQValue:(int)qValue {
    selectedType = qValue;
    self.qValueLabel.text = [NSString stringWithFormat:@"%d", qValue];
}
- (void)getBasebandSession:(AsP52NSessionType)session {
    NSString *sessionString = @"";
    switch (session) {
        case AsP52NSessionType_S0:{
            sessionString = @"Session0";
            selectedType = 0;
        }
            break;
        case AsP52NSessionType_S1:{
            sessionString = @"Session1";
            selectedType = 1;
        }
            break;
        case AsP52NSessionType_S2:{
            sessionString = @"Session2";
            selectedType = 2;
        }
            break;
        case AsP52NSessionType_S3:{
            sessionString = @"Session3";
            selectedType = 3;
        }
            break;
        default:
            break;
    }
    self.sessionLabel.text = sessionString;
}
- (void)getBasebandInventoryFlag:(AsP52NInventoryFlag)inventoryFlag {
    NSString *inventoryFlagString = @"";
    switch (inventoryFlag) {
        case AsP52NInventoryFlag_A:{
            inventoryFlagString = @"InventoryFlag_A";
            selectedType = 0;
        }
            break;
        case AsP52NInventoryFlag_B:{
            inventoryFlagString = @"InventoryFlag_B";
            selectedType = 1;
        }
            break;
        case AsP52NInventoryFlag_AB:{
            inventoryFlagString = @"InventoryFlag_AB";
            selectedType = 2;
        }
            break;
        default:
            break;
    }
    self.inventoryFlagLabel.text = inventoryFlagString;
}
@end
