//
//  FrequencyListViewController.m
//  P52NDemo
//
//  Created by mac on 2020/11/27.
//

#import "FrequencyListViewController.h"
#import <P52NSDK/P52NSDK.h>
#import <SVProgressHUD/SVProgressHUD.h>
#import "Utils.h"
@interface FrequencyListViewController ()<AsP52NDelegate, UIPickerViewDelegate, UIPickerViewDataSource>
@property (strong, nonatomic) IBOutlet UITableView *tableView;
@property (strong, nonatomic) IBOutlet UISegmentedControl *hopSegment;
@property (strong, nonatomic) UIPickerView *pickerView;
@property (strong, nonatomic) IBOutlet UILabel *frequencyLabel;
@end

@implementation FrequencyListViewController{
    NSArray *dataArray;
    NSMutableArray *dataListArray;
    AsP52N *asP52N;
    NSInteger _selected;
    BOOL pickViewStatus;
    NSString *_frequency;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    pickViewStatus = NO;
    _frequency = [[NSString alloc] init];
    dataArray = [NSArray arrayWithObjects: @"Gb920_925", @"FCC902_928", @"ETSI866_868", @"JP_916_8_920_8",nil];
    dataListArray = [[NSMutableArray alloc] init];
    self.tableView.tableFooterView = [[UIView alloc] initWithFrame:CGRectZero];
    self.frequencyLabel.layer.masksToBounds = YES;
    self.frequencyLabel.layer.borderColor = [[UIColor lightGrayColor] CGColor];
    self.frequencyLabel.layer.borderWidth = 1;
    self.frequencyLabel.layer.cornerRadius = 5;
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(frequencyAction)];
    self.frequencyLabel.userInteractionEnabled = YES;
    [self.frequencyLabel addGestureRecognizer:tap];
    self.pickerView = [[UIPickerView alloc]initWithFrame:CGRectMake(0, [UIScreen mainScreen].bounds.size.height - 300, [UIScreen mainScreen].bounds.size.width, 300)];
    self.pickerView.delegate = self;
    self.pickerView.dataSource = self;
    self.pickerView.hidden = YES;
    self.pickerView.backgroundColor = [UIColor whiteColor];
    [self.view addSubview:self.pickerView];
    asP52N = [AsP52N sharedInstance];
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"FRID frequency"];
    [asP52N getReaderRFIDFrequency];
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N receivedReaderRFIDFrequency:(AsP52NRFIDFrequencyStatus)frequency {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    if (frequency == AsP52NRFIDFrequencyStatus920_925MHz || frequency == AsP52NRFIDFrequencyStatusFCC || frequency == AsP52NRFIDFrequencyStatusETSI || frequency == AsP52NRFIDFrequencyStatusJP) {
        if (frequency == AsP52NRFIDFrequencyStatus920_925MHz) {
            _selected = 0;
        } else if (frequency == AsP52NRFIDFrequencyStatusFCC) {
            _selected = 1;
        } else if (frequency == AsP52NRFIDFrequencyStatusETSI) {
            _selected = 2;
        } else if (frequency == AsP52NRFIDFrequencyStatusJP) {
            _selected = 3;
        }
        [self.pickerView selectRow:_selected inComponent:0 animated:NO];
        self.frequencyLabel.text = [dataArray objectAtIndex:_selected];
        [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
        [SVProgressHUD showWithStatus:@"Frequency list"];
        dataListArray = [[NSMutableArray alloc] init];
        dataListArray = (NSMutableArray *)[asP52N getReaderRFIDFrequencyList:frequency];
        [self.tableView reloadData];
        [asP52N getReaderFrequency];
    }
}
- (void)AsP52N:(AsP52N *)AsP52N receivedSetRFIDFrequencyStatus:(AsP52NExecutionStatus)statusCode {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    if (AsP52NExecutionStatusSuccess == statusCode) {
        [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
        [SVProgressHUD showWithStatus:@"FRID frequency"];
        [asP52N getReaderRFIDFrequency];
    }else{
        NSString *failedString = [asP52N getMessageWithResultCode:statusCode];
        [Utils showAlert:@"" message:failedString];
    }
}
- (void)AsP52N:(AsP52N *)AsP52N receivedReaderFrequencyAutomatic:(BOOL)frequencyAutomatic frequencyList:(NSString *)frequencyList {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    if (frequencyAutomatic) {
        [self.hopSegment setSelectedSegmentIndex:0];
    } else {
        [self.hopSegment setSelectedSegmentIndex:1];
    }
    NSArray *array = [frequencyList componentsSeparatedByString:@","];
    for (NSString *string in array) {
        NSData *data = [self hexStringToData:string];
        Byte *byte = (Byte *)[data bytes];
        NSInteger i = byte[0];
        if (dataListArray.count > i) {
            NSMutableDictionary *dic = [dataListArray objectAtIndex:i];
            [dic setValue:@"1" forKey:[[dic allKeys] firstObject]];
        }
    }
    [self.tableView reloadData];
}
- (void)AsP52N:(AsP52N *)AsP52N receivedSetFrequencyStatus:(AsP52NExecutionStatus)statusCode {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    NSString *failedString = [asP52N getMessageWithResultCode:statusCode];
    [Utils showAlert:@"" message:failedString];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state != AsP52NetworkStateConnected) {
            [self.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}
#pragma mark - Action
- (IBAction)onClickDoneBarItem:(UIBarButtonItem *)sender {
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Set Frequency list"];
    BOOL status = NO;
    if (self.hopSegment.selectedSegmentIndex == 0) {
        status = YES;
    } else {
        status = NO;
    }
    NSMutableString *newString = [[NSMutableString alloc] init];
    for (int i = 0; i < dataListArray.count; i++) {
        NSMutableDictionary *dic = [dataListArray objectAtIndex:i];
        if ([[[dic allValues] firstObject] boolValue] == YES) {
            NSString *string = [NSString stringWithFormat:@"%d", i];
            if (newString.length > 0) {
                [newString appendString:[NSString stringWithFormat:@",%@",string]];
            } else {
                [newString appendString:[NSString stringWithFormat:@"%@",string]];
            }
        }
    }
    [asP52N setReaderFrequencyAutomatic:status frequencyList:(NSString *)newString];
}
- (IBAction)onClickHopSegment:(UISegmentedControl *)sender {

}
- (void)frequencyAction {
    if (pickViewStatus) {
        pickViewStatus = NO;
        self.pickerView.hidden = YES;
    } else {
        pickViewStatus = YES;
        self.pickerView.hidden = NO;
    }
}
#pragma mark - Table view data source
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return dataListArray.count;
}
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    NSString *cellIdentifier = @"tableViewCell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if(!cell){
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellIdentifier];
    }
    NSMutableDictionary *dic = [dataListArray objectAtIndex:indexPath.row];
    float value = [[[dic allKeys] firstObject] floatValue];
    cell.textLabel.text = [NSString stringWithFormat:@"%.3f", value];
    if ([[[dic allValues] firstObject] boolValue] == YES) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    } else {
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    return cell;
}
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    NSMutableDictionary *dic = [dataListArray objectAtIndex:indexPath.row];
    BOOL status = [[[dic allValues] firstObject] boolValue];
    if (status) {
        [dic setValue:@"0" forKey:[[dic allKeys] firstObject]];
    } else {
        [dic setValue:@"1" forKey:[[dic allKeys] firstObject]];
    }
    [self.tableView reloadData];
}
#pragma mark - UIPickerViewDataSource
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView{
    return 1;
}
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component{
    return dataArray.count;
}
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component{
    return [dataArray objectAtIndex:row];
}
- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component{
    _selected = row;
    pickViewStatus = NO;
    self.pickerView.hidden = YES;
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Set FRID frequency"];
    if (_selected == 0) {
        [asP52N setReaderRFIDFrequency:AsP52NRFIDFrequencyStatus920_925MHz];
    } else if (_selected == 1) {
        [asP52N setReaderRFIDFrequency:AsP52NRFIDFrequencyStatusFCC];
    } else if (_selected == 2) {
        [asP52N setReaderRFIDFrequency:AsP52NRFIDFrequencyStatusETSI];
    } else if (_selected == 3) {
        [asP52N setReaderRFIDFrequency:AsP52NRFIDFrequencyStatusJP];
    }
}
#pragma mark - Private
- (void)dismissHUD {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [SVProgressHUD dismiss];
    }];
}
- (NSData *)hexStringToData:(NSString *)str {
    if (!str || [str length] == 0) {
        return nil;
    }
    NSMutableData *hexData = [[NSMutableData alloc] initWithCapacity:8];
    NSRange range;
    if ([str length] % 2 == 0) {
        range = NSMakeRange(0, 2);
    } else {
        range = NSMakeRange(0, 1);
    }
    for (NSInteger i = range.location; i < [str length]; i += 2) {
        unsigned int anInt;
        NSString *hexCharStr = [str substringWithRange:range];
        NSScanner *scanner = [[NSScanner alloc] initWithString:hexCharStr];

        [scanner scanHexInt:&anInt];
        NSData *entity = [[NSData alloc] initWithBytes:&anInt length:1];
        [hexData appendData:entity];

        range.location += range.length;
        range.length = 2;
    }
    return hexData;
}

@end
