//
//  TagLogViewController.m
//  P52NDemo
//
//  Created by mac on 2020/11/27.
//

#import "TagLogViewController.h"
#import <P52NSDK/P52NSDK.h>
#import <SVProgressHUD/SVProgressHUD.h>
#import "Utils.h"
@interface TagLogViewController ()<AsP52NDelegate, UITextFieldDelegate>
@property (strong, nonatomic) IBOutlet UITextField *filterTextField;
@property (strong, nonatomic) IBOutlet UITextField *rssiTextField;
@end

@implementation TagLogViewController{
    AsP52N *asP52N;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    asP52N = [AsP52N sharedInstance];
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Filter"];
    [asP52N getReaderTagLog];
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N receivedReaderTagLogFilterTime:(int)filtertime RSSI:(int)rssi {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    self.filterTextField.text = [NSString stringWithFormat:@"%d", filtertime];
    self.rssiTextField.text = [NSString stringWithFormat:@"%d", -1 * rssi];
}
- (void)AsP52N:(AsP52N *)AsP52N receivedSetTagLogStatus:(AsP52NExecutionStatus)statusCode {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    NSString *failedString = [asP52N getMessageWithResultCode:statusCode];
    [Utils showAlert:@"" message:failedString];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state != AsP52NetworkStateConnected) {
            [self.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}
#pragma mark - Action
- (IBAction)onClickDoneBarItem:(UIBarButtonItem *)sender {
    [self.view endEditing:YES];
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Set filter"];
    [asP52N setReaderTagLogFilterTime:[self.filterTextField.text intValue] RSSI:-1 * [self.rssiTextField.text intValue]];
}
#pragma mark - UITextFieldDelegate
- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    if (textField == self.filterTextField) {
        if ([[textField.text stringByAppendingString:string] intValue] < 0 || [[textField.text stringByAppendingString:string] intValue] > 65535) {
            return NO;
        } else {
            return YES;
        }
    } else if (textField == self.rssiTextField) {
        if ([[textField.text stringByAppendingString:string] intValue] < 0 || [[textField.text stringByAppendingString:string] intValue] > 110) {
            return NO;
        } else {
            return YES;
        }
    }
    return YES;
}
#pragma mark - System
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
}
#pragma mark - Private
- (void)dismissHUD {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [SVProgressHUD dismiss];
    }];
}

@end
