//
//  KillViewController.m
//  P52NDemo
//
//  Created by mac on 2020/12/11.
//

#import "KillViewController.h"
#import <P52NSDK/P52NSDK.h>
#import <SVProgressHUD/SVProgressHUD.h>
#import "Utils.h"
#import "System.h"

@interface KillViewController ()<AsP52NDelegate>
@property (strong, nonatomic) IBOutlet UITextField *passwordTextField;
@property (strong, nonatomic) IBOutlet UISegmentedControl *matchSegment;
@property (strong, nonatomic) IBOutlet UISwitch *matchSwitch;
@property (strong, nonatomic) IBOutlet UITextField *matchTextField;
@property (strong, nonatomic) IBOutlet UITextField *startAddressTextField;
@end

@implementation KillViewController
{
    AsP52N *asP52N;
    AsP52NMemoryBankType memoryBankType;
    System *system;
}
- (void)viewDidLoad {
    [super viewDidLoad];
    self.matchTextField.text = _epcData.epc;
    if ([self.matchTextField.text isEqualToString:@""]) {
        self.matchTextField.enabled = NO;
    } else {
        self.matchTextField.enabled = YES;
    }
    system = [System sharedSystem];
    asP52N = [AsP52N sharedInstance];
    memoryBankType = AsP52NMemoryBankTypeReserved;
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
}
#pragma mark - Action
- (IBAction)onClickMatchSegment:(UISegmentedControl *)sender {
    switch (sender.selectedSegmentIndex) {
        case 0:
        {
            self.startAddressTextField.text = @"2";
            self.matchTextField.text = _epcData.epc;
        }
            break;
        case 1:
        {
            self.startAddressTextField.text = @"0";
            self.matchTextField.text = _epcData.tid;
        }
            break;
        case 2:
        {
            self.startAddressTextField.text = @"0";
            self.matchTextField.text = _epcData.userData;
        }
            break;
        default:
            break;
    }
    if ([self.matchTextField.text isEqualToString:@""]) {
        self.matchTextField.enabled = NO;
    } else {
        self.matchTextField.enabled = YES;
    }
}
- (IBAction)onClickKillButton:(UIBarButtonItem *)sender {
    [self.view endEditing:YES];
    if ([self.passwordTextField.text isEqualToString:@""]) {
        [Utils showAlert:@"Error" message:@"Please enter your password"];
        return;
    } else if (self.passwordTextField.text.length != 8) {
        [Utils showAlert:@"Error" message:@"Please enter your eight-digit password"];
        return;
    }
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Kill?" message:nil preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *ok = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
        [SVProgressHUD showWithStatus:@"Kill data"];
        [self->asP52N killTagWithAntenna1:[self->system getAnt1Status]
                                 antenna2:[self->system getAnt2Status]
                                 antenna3:[self->system getAnt3Status]
                                 antenna4:[self->system getAnt4Status]
                                 antenna5:[self->system getAnt5Status]
                                 antenna6:[self->system getAnt6Status]
                                 antenna7:[self->system getAnt7Status]
                                 antenna8:[self->system getAnt8Status]
                             killPassword:self.passwordTextField.text
                              isUsedMatch:self.matchSwitch.on
                      matchMemoryBankType:(AsP52NMemoryBankType)(self.matchSegment.selectedSegmentIndex + 1)
                                matchData:self.matchTextField.text
                        matchStartAddress:[self.startAddressTextField.text intValue]];
        
    }];
    UIAlertAction *cancel = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil];
    [alert addAction:ok];
    [alert addAction:cancel];
    [self presentViewController:alert animated:YES completion:nil];
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N receivedKillDataStatus:(AsP52NExecutionStatus)statuCodes {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    NSString *failedString = [asP52N getMessageWithResultCode:statuCodes];
    [Utils showAlert:@"" message:failedString];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state != AsP52NetworkStateConnected) {
            [self.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}
#pragma mark - System
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
}
#pragma mark - Private
- (void)viewBack {
    [self.view endEditing:YES];
}
- (void)dismissHUD {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [SVProgressHUD dismiss];
    }];
}
@end
