//
//  LockViewController.m
//  P52NDemo
//
//  Created by mac on 2020/12/11.
//

#import "LockViewController.h"
#import <P52NSDK/P52NSDK.h>
#import <SVProgressHUD/SVProgressHUD.h>
#import "Utils.h"
#import "System.h"

@interface LockViewController ()<AsP52NDelegate, UIPickerViewDelegate, UIPickerViewDataSource>
@property (strong, nonatomic) IBOutlet UILabel *lockLabel;
@property (strong, nonatomic) IBOutlet UILabel *operationLabel;
@property (strong, nonatomic) IBOutlet UISegmentedControl *matchSegment;
@property (strong, nonatomic) IBOutlet UISwitch *matchSwitch;
@property (strong, nonatomic) IBOutlet UITextField *matchTextField;
@property (strong, nonatomic) IBOutlet UITextField *startAddressTextField;
@property (strong, nonatomic) UIPickerView *lockPickerView;
@property (strong, nonatomic) UIPickerView *operatioPickerView;
@property (strong, nonatomic) IBOutlet UITextField *passwordTextField;
@property (strong, nonatomic) IBOutlet UIBarButtonItem *lockBarItem;
@end

@implementation LockViewController
{
    AsP52N *asP52N;
    AsP52NMemoryBankType memoryBankType;
    System *system;
    NSArray *lockArray;
    NSArray *operationArray;
    BOOL lockStatus;
    BOOL operationStatus;
    AsP52NTargetMemoryType _lockSelected;
    AsP52NLockActionType _operationSelected;
}
- (void)viewDidLoad {
    [super viewDidLoad];
    lockStatus = NO;
    operationStatus = NO;
    _lockSelected = AsP52NTargetMemoryTypeKill;
    _operationSelected = AsP52NLockActionTypeUnlock;
    self.matchTextField.text = _epcData.epc;
    if ([self.matchTextField.text isEqualToString:@""]) {
        self.matchTextField.enabled = NO;
    } else {
        self.matchTextField.enabled = YES;
    }
    self.lockLabel.layer.masksToBounds = YES;
    self.lockLabel.layer.borderColor = [[UIColor colorWithRed:229 / 255.f green:229 / 255.f blue:234 / 255.f alpha:1] CGColor];
    self.lockLabel.layer.borderWidth = 1;
    self.lockLabel.layer.cornerRadius = 5;

    self.operationLabel.layer.masksToBounds = YES;
    self.operationLabel.layer.borderColor = [[UIColor colorWithRed:229 / 255.f green:229 / 255.f blue:234 / 255.f alpha:1] CGColor];
    self.operationLabel.layer.borderWidth = 1;
    self.operationLabel.layer.cornerRadius = 5;

    self.lockLabel.userInteractionEnabled = YES;
    UITapGestureRecognizer *lockLabelTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(lock)];
    [self.lockLabel addGestureRecognizer:lockLabelTap];

    self.operationLabel.userInteractionEnabled = YES;
    UITapGestureRecognizer *operationTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(operation)];
    [self.operationLabel addGestureRecognizer:operationTap];

    system = [System sharedSystem];
    asP52N = [AsP52N sharedInstance];
    memoryBankType = AsP52NMemoryBankTypeReserved;
    lockArray = [NSArray arrayWithObjects: @"Kill",
                                           @"Access",
                                           @"EPC",
                                           @"TID",
                                           @"User", nil];
    operationArray = [NSArray arrayWithObjects: @"Unlock",
                                                @"Lock",
                                                @"PermaUnlock",
                                                @"PermaLock", nil];
    self.lockPickerView = [[UIPickerView alloc]initWithFrame:CGRectMake(0, [UIScreen mainScreen].bounds.size.height - 200 - 64, [UIScreen mainScreen].bounds.size.width, 200)];
    self.lockPickerView.delegate = self;
    self.lockPickerView.dataSource = self;
    self.lockPickerView.hidden = YES;
    self.lockPickerView.backgroundColor = [UIColor whiteColor];
    [self.view addSubview:self.lockPickerView];
    [self.lockPickerView selectRow:_lockSelected inComponent:0 animated:NO];

    self.operatioPickerView = [[UIPickerView alloc]initWithFrame:CGRectMake(0, [UIScreen mainScreen].bounds.size.height - 200 - 64, [UIScreen mainScreen].bounds.size.width, 200)];
    self.operatioPickerView.delegate = self;
    self.operatioPickerView.dataSource = self;
    self.operatioPickerView.hidden = YES;
    self.operatioPickerView.backgroundColor = [UIColor whiteColor];
    [self.view addSubview:self.operatioPickerView];
    [self.operatioPickerView selectRow:_operationSelected inComponent:0 animated:NO];
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
}
#pragma mark - Action
- (IBAction)onClickMatchSegment:(UISegmentedControl *)sender {
    switch (sender.selectedSegmentIndex) {
        case 0:
        {
            self.startAddressTextField.text = @"2";
            self.matchTextField.text = _epcData.epc;
        }
            break;
        case 1:
        {
            self.startAddressTextField.text = @"0";
            self.matchTextField.text = _epcData.tid;
        }
            break;
        case 2:
        {
            self.startAddressTextField.text = @"0";
            self.matchTextField.text = _epcData.userData;
        }
            break;
        default:
            break;
    }
    if ([self.matchTextField.text isEqualToString:@""]) {
        self.matchTextField.enabled = NO;
    } else {
        self.matchTextField.enabled = YES;
    }
}
- (IBAction)onClickLockBarItem:(UIBarButtonItem *)sender {
    [self.view endEditing:YES];
    if ([self.passwordTextField.text isEqualToString:@""]) {
        [Utils showAlert:@"Error" message:@"Please enter your password"];
        return;
    } else if (self.passwordTextField.text.length != 8) {
        [Utils showAlert:@"Error" message:@"Please enter your eight-digit password"];
        return;
    }
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    if (_operationSelected == AsP52NLockActionTypeLock || _operationSelected == AsP52NLockActionTypePermaLock) {
        [SVProgressHUD showWithStatus:@"Lock data"];
    } else {
        [SVProgressHUD showWithStatus:@"UnLock data"];
    }
    [asP52N lockTagWithAntenna1:[system getAnt1Status]
                       antenna2:[system getAnt2Status]
                       antenna3:[system getAnt3Status]
                       antenna4:[system getAnt4Status]
                       antenna5:[system getAnt5Status]
                       antenna6:[system getAnt6Status]
                       antenna7:[system getAnt7Status]
                       antenna8:[system getAnt8Status]
                   targetMemory:_lockSelected
                     lockAction:_operationSelected
                    isUsedMatch:self.matchSwitch.on
            matchMemoryBankType:(AsP52NMemoryBankType)(self.matchSegment.selectedSegmentIndex + 1)
                      matchData:self.matchTextField.text
              matchStartAddress:[self.startAddressTextField.text intValue]
                 accessPassword:self.passwordTextField.text];
}
- (void)lock {
    if (lockStatus) {
        lockStatus = NO;
        self.lockPickerView.hidden = YES;
    } else {
        lockStatus = YES;
        self.lockPickerView.hidden = NO;
    }
}
- (void)operation {
    if (operationStatus) {
        operationStatus = NO;
        self.operatioPickerView.hidden = YES;
    } else {
        operationStatus = YES;
        self.operatioPickerView.hidden = NO;
    }
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N receivedLockDataStatus:(AsP52NExecutionStatus)statusCode {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    NSString *failedString = [asP52N getMessageWithResultCode:statusCode];
    [Utils showAlert:@"" message:failedString];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state != AsP52NetworkStateConnected) {
            [self.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}
#pragma mark - UIPickerViewDataSource
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView{
    return 1;
}
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component{
    if (pickerView == self.lockPickerView) {
        return lockArray.count;
    } else {
        return operationArray.count;
    }
}
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component{
    if (pickerView == self.lockPickerView) {
        return [lockArray objectAtIndex:row];
    } else {
        return [operationArray objectAtIndex:row];
    }
}
- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component{
    if (pickerView == self.lockPickerView) {
        [self lock];
        _lockSelected = (AsP52NTargetMemoryType)row;
        self.lockLabel.text = [lockArray objectAtIndex:row];
    } else {
        [self operation];
        _operationSelected = (AsP52NLockActionType)row;
        self.operationLabel.text = [operationArray objectAtIndex:row];
        if (_operationSelected == AsP52NLockActionTypeLock || _operationSelected == AsP52NLockActionTypePermaLock) {
            [self.lockBarItem setTitle:@"Lock"];
        } else {
            [self.lockBarItem setTitle:@"UnLock"];
        }
    }
}
#pragma mark - System
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
}
#pragma mark - Private
- (void)viewBack {
    lockStatus = NO;
    self.lockPickerView.hidden = YES;
    operationStatus = NO;
    self.operatioPickerView.hidden = YES;
    [self.view endEditing:YES];
}
- (void)dismissHUD {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [SVProgressHUD dismiss];
    }];
}
@end
