//
//  ReadViewController.m
//  P52NDemo
//
//  Created by mac on 2020/12/9.
//

#import "ReadViewController.h"
#import <P52NSDK/P52NSDK.h>
#import <SVProgressHUD/SVProgressHUD.h>
#import "Utils.h"
#import "System.h"

@interface ReadViewController ()<AsP52NDelegate>
@property (strong, nonatomic) IBOutlet UIScrollView *backScroView;
@property (strong, nonatomic) IBOutlet UISegmentedControl *tidSegment;
@property (strong, nonatomic) IBOutlet UISegmentedControl *readSegment;
@property (strong, nonatomic) IBOutlet UITextField *epcOffsetTextField;
@property (strong, nonatomic) IBOutlet UITextField *epcLenghtTextField;
@property (strong, nonatomic) IBOutlet UITextField *passWordTextField;
@property (strong, nonatomic) IBOutlet UISwitch *monzaSwitch;
@property (strong, nonatomic) IBOutlet UISwitch *rfmicronSwitch;
@property (strong, nonatomic) IBOutlet UISwitch *emsensorSwitch;
@property (strong, nonatomic) IBOutlet UISwitch *matchSwitch;
@property (strong, nonatomic) IBOutlet UISegmentedControl *matchSegment;
@property (strong, nonatomic) IBOutlet UITextField *matchAddressTextField;
@property (strong, nonatomic) IBOutlet UITextField *matchTextField;
@property (strong, nonatomic) IBOutlet UITextView *readTextView;
@end

@implementation ReadViewController
{
    AsP52N *asP52N;
    AsP52NMemoryBankType memoryBankType;
    System *system;
}
- (void)viewDidLoad {
    [super viewDidLoad];
    self.matchTextField.text = _epcData.epc;
    if ([self.matchTextField.text isEqualToString:@""]) {
        self.matchTextField.enabled = NO;
    } else {
        self.matchTextField.enabled = YES;
    }
    self.tidSegment.enabled = NO;
    self.backScroView.contentSize = CGSizeMake(0, self.emsensorSwitch.frame.origin.y + self.emsensorSwitch.frame.size.height + 100);
    self.readTextView.userInteractionEnabled = NO;
    self.readTextView.layer.masksToBounds = YES;
    self.readTextView.layer.borderColor = [[UIColor lightGrayColor] CGColor];
    self.readTextView.layer.borderWidth = 1;
    self.readTextView.layer.cornerRadius = 5;
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(viewBack)];
    [self.backScroView addGestureRecognizer:tap];
    system = [System sharedSystem];
    asP52N = [AsP52N sharedInstance];
    memoryBankType = AsP52NMemoryBankTypeReserved;
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
}
#pragma mark - Action
- (IBAction)onClickDoneBarItem:(UIBarButtonItem *)sender {
    [self.view endEditing:YES];
    if (![self.passWordTextField.text isEqualToString:@""] && self.passWordTextField.text.length != 8) {
        [Utils showAlert:@"Error" message:@"Please enter your eight-digit password"];
        return;
    }
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Read data"];
    BOOL reservedStatus = NO;
    BOOL epcStatus = NO;
    BOOL tidStatus = NO;
    BOOL userDataStatus = NO;
    switch (memoryBankType) {
        case AsP52NMemoryBankTypeReserved:{
            reservedStatus = YES;
        }
            break;
        case AsP52NMemoryBankTypeEPC:{
            epcStatus = YES;
        }
            break;
        case AsP52NMemoryBankTypeTID:{
            tidStatus = YES;
        }
            break;
        case AsP52NMemoryBankTypeUser:{
            userDataStatus = YES;
        }
            break;
        default:
            break;
    }
    BOOL passwordStatus = NO;
    if (![self.passWordTextField.text isEqualToString:@""]) {
        passwordStatus = YES;
    }
    [asP52N readDataWithAntenna1:[system getAnt1Status] antenna2:[system getAnt2Status] antenna3:[system getAnt3Status] antenna4:[system getAnt4Status] antenna5:[system getAnt5Status] antenna6:[system getAnt6Status] antenna7:[system getAnt7Status] antenna8:[system getAnt8Status] isContinueScan:NO isUsedTID:tidStatus isUsedTidLength:self.tidSegment.selectedSegmentIndex  tidLength:[self.epcLenghtTextField.text intValue] isUsedUserData:userDataStatus userDataOffset:[self.epcOffsetTextField.text intValue] userDataLength:[self.epcLenghtTextField.text intValue] isUsedReserved:reservedStatus reservedOffset:[self.epcOffsetTextField.text intValue] reservedLength:[self.epcLenghtTextField.text intValue] isUsedPassword:passwordStatus accessPassword:self.passWordTextField.text isUsedMonza:self.monzaSwitch.on isUsedRfmicron:self.rfmicronSwitch.on isUsedEMSensor:self.emsensorSwitch.on isUsedEPC:epcStatus epcOffset:[self.epcOffsetTextField.text intValue] epcLength:[self.epcLenghtTextField.text intValue] isUsedMatch:self.matchSwitch.on matchMemoryBankType:(AsP52NMemoryBankType)(self.matchSegment.selectedSegmentIndex + 1) matchData:self.matchTextField.text matchStartAddress:[self.matchAddressTextField.text intValue]];
}
- (IBAction)onClickMatchSegment:(UISegmentedControl *)sender {
    switch (sender.selectedSegmentIndex) {
        case 0:
        {
            self.matchAddressTextField.text = @"2";
            self.matchTextField.text = _epcData.epc;
        }
            break;
        case 1:
        {
            self.matchAddressTextField.text = @"0";
            self.matchTextField.text = _epcData.tid;
        }
            break;
        case 2:
        {
            self.matchAddressTextField.text = @"0";
            self.matchTextField.text = _epcData.userData;
        }
            break;
        default:
            break;
    }
    if ([self.matchTextField.text isEqualToString:@""]) {
        self.matchTextField.enabled = NO;
    } else {
        self.matchTextField.enabled = YES;
    }
}
- (IBAction)onClickReadSegment:(UISegmentedControl *)sender {
    switch (sender.selectedSegmentIndex) {
        case 0:
        {
            memoryBankType = AsP52NMemoryBankTypeReserved;
            self.tidSegment.enabled = NO;
            self.epcOffsetTextField.enabled = YES;
            self.epcOffsetTextField.text = @"0";
            self.epcLenghtTextField.text = @"0";
        }
            break;
        case 1:
        {
            memoryBankType = AsP52NMemoryBankTypeEPC;
            self.tidSegment.enabled = NO;
            self.epcOffsetTextField.enabled = YES;
            self.epcOffsetTextField.text = @"2";
            self.epcLenghtTextField.text = @"0";
        }
            break;
        case 2:
        {
            memoryBankType =AsP52NMemoryBankTypeTID;
            self.tidSegment.enabled = YES;
            self.epcOffsetTextField.enabled = NO;
            self.epcOffsetTextField.text = @"0";
            self.epcLenghtTextField.text = @"6";
        }
            break;
        case 3:
        {
            memoryBankType = AsP52NMemoryBankTypeUser;
            self.tidSegment.enabled = NO;
            self.epcOffsetTextField.enabled = YES;
            self.epcOffsetTextField.text = @"0";
            self.epcLenghtTextField.text = @"0";
        }
            break;
        default:
            break;
    }
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N receivedInventoryCustomDataWithRSSI:(NSString *)RSSI channel:(NSString *)channel phase:(NSString *)phase antenna:(NSString *)antenna PC:(NSString *)PC EPC:(NSString *)EPC TID:(NSString *)TID userData:(NSString *)userData reserved:(NSString *)reserved epcBank:(NSString *)epcBank moisture:(NSString *)moisture temperature:(NSString *)temperature{
    [asP52N stopInventory];
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    NSLog (@"APP epcData = %@, pcData = %@, ANT = %@, phase = %@,channel = %@, RSSI = %@, userData = %@, reserved = %@, epcBank = %@, moisture = %@, temperature = %@",EPC,PC,antenna,phase,channel,RSSI,userData,reserved,epcBank,moisture,temperature);
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        NSString *string = @"";
        switch (self.readSegment.selectedSegmentIndex) {
            case 0:
            {
                string = reserved;
            }
                break;
            case 1:
            {
                string = epcBank;
            }
                break;
            case 2:
            {
                string = TID;
            }
                break;
            case 3:
            {
                string = userData;
            }
                break;
            default:
                break;
        }
        self.readTextView.text = string;

    }];
}
- (void)AsP52N:(AsP52N *)AsP52N receivedInventoryStatus:(AsP52NExecutionStatus)statusCode {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    NSString *failedString = [asP52N getMessageWithResultCode:statusCode];
    [Utils showAlert:@"" message:failedString];
}
- (void)AsP52N:(AsP52N *)AsP52N receivedReadDataStatus:(AsP52NExecutionStatus)statusCode {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    NSString *failedString = [asP52N getMessageWithResultCode:statusCode];
    [Utils showAlert:@"" message:failedString];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state != AsP52NetworkStateConnected) {
            [self.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}
-(void)AsP52N:(AsP52N *)AsP52N didInventoryEndExecutionStatus:(AsP52NExecutionStatus)statusCode{
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [SVProgressHUD dismiss];
    }];
}
#pragma mark - Private
- (void)viewBack {
    [self.backScroView endEditing:YES];
}
- (void)dismissHUD {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [self->asP52N stopInventory];
        [SVProgressHUD dismiss];
    }];
}
@end
