//
//  WriteViewController.m
//  P52NDemo
//
//  Created by mac on 2020/12/4.
//

#import "WriteViewController.h"
#import <SVProgressHUD/SVProgressHUD.h>
#import <P52NSDK/P52NSDK.h>
#import "System.h"
#import "Utils.h"

@interface WriteViewController ()<AsP52NDelegate, UITextViewDelegate>
@property (strong, nonatomic) IBOutlet UISegmentedControl *currentTypeSegment;
@property (strong, nonatomic) IBOutlet UITextField *currentDataTextField;
@property (strong, nonatomic) IBOutlet UITextField *startAddressTextField;
@property (strong, nonatomic) IBOutlet UISegmentedControl *writeTypeSegment;
@property (strong, nonatomic) IBOutlet UITextView *writeTextView;
@property (strong, nonatomic) IBOutlet UITextField *startWriteAddressTextField;
@property (strong, nonatomic) IBOutlet UITextField *passwordTextField;
@property (strong, nonatomic) IBOutlet UITextField *lenghtTextField;
@property (strong, nonatomic) IBOutlet UISwitch *matchSwitch;
@end

@implementation WriteViewController
{
    AsP52N *asP52N;
    System *system;
}
- (void)viewDidLoad {
    [super viewDidLoad];
    system = [System sharedSystem];
    asP52N = [AsP52N sharedInstance];
    self.writeTextView.layer.masksToBounds = YES;
    self.writeTextView.layer.borderColor = [[UIColor lightGrayColor] CGColor];
    self.writeTextView.layer.borderWidth = 1;
    self.writeTextView.layer.cornerRadius = 5;
    self.currentDataTextField.text = _epcData.epc;
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N receivedWriteDataStatus:(AsP52NExecutionStatus)statusCode {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(dismissHUD) object:nil];
    [SVProgressHUD dismiss];
    NSString *failedString = [asP52N getMessageWithResultCode:statusCode];
    [Utils showAlert:@"" message:failedString];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state != AsP52NetworkStateConnected) {
            [self.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}
#pragma mark - UITextViewDelegate
- (void)textViewDidChange:(UITextView *)textView {
    NSString *string = textView.text;
    if ((int)string.length % 4 > 0) {
        self.lenghtTextField.text = [NSString stringWithFormat:@"%d", (int)string.length / 4 + 1];
    } else {
        self.lenghtTextField.text = [NSString stringWithFormat:@"%d", (int)string.length / 4];
    }
}
#pragma mark - Action
- (IBAction)onClickDoneBarItem:(UIBarButtonItem *)sender {
    [self.view endEditing:YES];
    if ([self.writeTextView.text isEqualToString:@""] && [self.startWriteAddressTextField.text isEqualToString:@""]) {
        [Utils showAlert:@"Error" message:@"Please enter write value"];
        return;
    }
    if (![self.passwordTextField.text isEqualToString:@""] && self.passwordTextField.text.length != 8) {
        [Utils showAlert:@"Error" message:@"Please enter your eight-digit password"];
        return;
    }
    [self performSelector:@selector(dismissHUD) withObject:nil afterDelay:10];
    [SVProgressHUD showWithStatus:@"Write data"];
    [asP52N writeDataWithAntenna1:[system getAnt1Status]
                         antenna2:[system getAnt2Status]
                         antenna3:[system getAnt3Status]
                         antenna4:[system getAnt4Status]
                         antenna5:[system getAnt5Status]
                         antenna6:[system getAnt6Status]
                         antenna7:[system getAnt7Status]
                         antenna8:[system getAnt8Status]
              writeMemoryBankType:(AsP52NMemoryBankType)self.currentTypeSegment.selectedSegmentIndex
                        writeData:self.writeTextView.text
                writeStartAddress:[self.startWriteAddressTextField.text intValue]
                      isUsedMatch:self.matchSwitch.on
              matchMemoryBankType:(AsP52NMemoryBankType)(self.writeTypeSegment.selectedSegmentIndex + 1)
                        matchData:self.currentDataTextField.text
                matchStartAddress:[self.startAddressTextField.text intValue]
                   accessPassword:self.passwordTextField.text];
}
- (IBAction)onClickCurrentTypeSegment:(UISegmentedControl *)sender {
    switch (sender.selectedSegmentIndex) {
        case 0:
        {
            self.currentDataTextField.text = _epcData.epc;
            self.startAddressTextField.text = @"2";
        }
            break;
        case 1:
        {
            self.currentDataTextField.text = _epcData.tid;
            self.startAddressTextField.text = @"0";
        }
            break;
        case 2:
        {
            self.currentDataTextField.text = _epcData.userData;
            self.startAddressTextField.text = @"0";
        }
            break;
        default:
            break;
    }
}
- (IBAction)onClickWritedTypeSegment:(UISegmentedControl *)sender {
    switch (sender.selectedSegmentIndex) {
        case 0:
        {
            self.startWriteAddressTextField.text = @"0";
        }
            break;
        case 1:
        {
            self.startWriteAddressTextField.text = @"2";
        }
            break;
        case 2:
        {
            self.startWriteAddressTextField.text = @"0";
        }
            break;
        case 3:
        {
            self.startWriteAddressTextField.text = @"0";
        }
            break;
        default:
            break;
    }
}
#pragma mark - System
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
}
#pragma mark - Private
- (void)dismissHUD {
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        [SVProgressHUD dismiss];
    }];
}
@end
