//
//  ViewController.m
//  P52NDemo
//
//  Created by MRX_ZYL on 13/10/2020.
//

#import "ViewController.h"
#import <P52NSDK/P52NSDK.h>
#import <SVProgressHUD/SVProgressHUD.h>
#import "EPCData.h"
#import "System.h"
#import "EPCDataCell.h"
#import "TIDDataCell.h"
#import "CustomDataCell.h"
#import "Utils.h"
#import "DataListController.h"

@interface ViewController ()<AsP52NDelegate,UITableViewDataSource,UITableViewDelegate>
@property (weak, nonatomic) IBOutlet UITextField *ipTextField;
@property (weak, nonatomic) IBOutlet UITextField *portTextField;
@property (weak, nonatomic) IBOutlet UIButton *connectButton;
@property (weak, nonatomic) IBOutlet UIButton *disConnectButton;
@property (weak, nonatomic) IBOutlet UILabel *countLabel;
@property (weak, nonatomic) IBOutlet UILabel *totalLabel;
@property (weak, nonatomic) IBOutlet UIButton *startButton;
@property (weak, nonatomic) IBOutlet UIButton *stopButton;
@property (weak, nonatomic) IBOutlet UIButton *clearButton;
@property (weak, nonatomic) IBOutlet UIView *backGroundView;
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *settingBarItem;
@property (weak, nonatomic) IBOutlet UILabel *versionLabel;
@property (weak, nonatomic) IBOutlet UILabel *temperatureLabel;

@end
@implementation ViewController{
    AsP52N *asP52N;
    System *system;
    NSMutableArray *dataArray;
    NSMutableDictionary *dataDic;
    NSMutableArray *arrIPs;
    int count;
    int total;
    BOOL searchStatus;
    EPCData *_epcData;
    NSTimer *getTemperatureTimer;
    BOOL inventoryStatus;
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    asP52N.delegate = self;
    if ([system getInventoryType] == InventoryTypeEPC) {
        self.tableView.rowHeight = 90.f;
    } else {
        self.tableView.rowHeight = 120.f;
    }
    if (!asP52N.isConnected) {
        [self setDisconnectUI];
    }
    [self startTemperatureTimer];
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:YES];
    asP52N.delegate = nil;
    if (asP52N && asP52N.isConnected) {
        [asP52N stopInventory];
        self.stopButton.backgroundColor = [UIColor lightGrayColor];
        self.stopButton.userInteractionEnabled = NO;
        self.startButton.backgroundColor = [UIColor redColor];
        self.startButton.userInteractionEnabled = YES;
    }
    [self stopTemperatureTimer];
}
- (void)startTemperatureTimer{
    [self getTemperature];
    if (getTemperatureTimer) {
        [getTemperatureTimer invalidate];
        getTemperatureTimer = nil;
    }
    getTemperatureTimer = [NSTimer scheduledTimerWithTimeInterval:5.0 target:self selector:@selector(getTemperature) userInfo:nil repeats:YES];
    [[NSRunLoop currentRunLoop] addTimer:getTemperatureTimer forMode:NSRunLoopCommonModes];
}
- (void)getTemperature{
    if([self->asP52N isConnected]){
        [self->asP52N getTemperature];
    }
}
- (void)stopTemperatureTimer{
    if (getTemperatureTimer) {
        [getTemperatureTimer invalidate];
        getTemperatureTimer = nil;
    }
}
- (void)viewDidLoad {
    [super viewDidLoad];
    system = [System sharedSystem];
    asP52N = [AsP52N sharedInstance];
    count = 0;
    total = 0;
    searchStatus = NO;
    inventoryStatus = NO;
    dataArray = [[NSMutableArray alloc] init];
    dataDic = [[NSMutableDictionary alloc] init];
    arrIPs  = [[NSMutableArray alloc]init];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(restartDevice) name:@"RestartDevice" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(factoryRest) name:@"FactoryRest" object:nil];
    [self setUI];
}
- (void)setUI {
    if (@available(iOS 15.0, *)){
        UINavigationBarAppearance *appearance = [[UINavigationBarAppearance alloc] init];
        appearance.backgroundEffect = [UIBlurEffect effectWithStyle:UIBlurEffectStyleRegular];
        appearance.backgroundColor = [UIColor whiteColor];
        appearance.shadowColor = [UIColor lightGrayColor];
        self.navigationController.navigationBar.scrollEdgeAppearance = appearance;
        self.navigationController.navigationBar.standardAppearance = appearance;
    }
    [SVProgressHUD setDefaultMaskType:SVProgressHUDMaskTypeBlack];
    self.tableView.rowHeight = 90.f;
    self.backGroundView.layer.masksToBounds = YES;
    self.backGroundView.layer.borderWidth = 5;
    self.backGroundView.layer.borderColor = [[UIColor blackColor] CGColor];
    self.backGroundView.layer.cornerRadius = 1;
    self.connectButton.layer.masksToBounds = YES;
    self.connectButton.layer.cornerRadius = 5;
    self.disConnectButton.layer.masksToBounds = YES;
    self.disConnectButton.layer.cornerRadius = 5;
    self.startButton.layer.masksToBounds = YES;
    self.startButton.layer.cornerRadius = 5;
    self.stopButton.layer.masksToBounds = YES;
    self.stopButton.layer.cornerRadius = 5;
    self.clearButton.layer.masksToBounds = YES;
    self.clearButton.layer.cornerRadius = 5;
    self.settingBarItem.enabled = NO;
    self.ipTextField.userInteractionEnabled = YES;
    self.portTextField.userInteractionEnabled = YES;
    self.connectButton.userInteractionEnabled = YES;
    self.disConnectButton.userInteractionEnabled = NO;
    self.disConnectButton.backgroundColor = [UIColor lightGrayColor];
    self.startButton.userInteractionEnabled = NO;
    self.startButton.backgroundColor = [UIColor lightGrayColor];
    self.stopButton.userInteractionEnabled = NO;
    self.stopButton.backgroundColor = [UIColor lightGrayColor];
    self.clearButton.userInteractionEnabled = NO;
    self.clearButton.backgroundColor = [UIColor lightGrayColor];
    self.tableView.userInteractionEnabled = NO;
    self.ipTextField.text = [system getIp];
    self.portTextField.text = [system getPort];
    NSDictionary *infoDictionary = [[NSBundle mainBundle] infoDictionary];
    NSString *app_Version = [infoDictionary objectForKey:@"CFBundleShortVersionString"];
    NSString *app_Build = [infoDictionary objectForKey:@"CFBundleVersion"];
    self.versionLabel.text = [NSString stringWithFormat:@"Version:%@ Build:%@", app_Version, app_Build];
    self.versionLabel.text = [NSString stringWithFormat:@"SDK Version:%@  APP Version:%@  Build:%@", [asP52N getSDKVersion], app_Version, app_Build];
}
#pragma mark - Action
- (IBAction)onClickSearchButton:(id)sender {
    NSLog(@"startSearchDevice");
    [self.view endEditing:YES];
    [arrIPs removeAllObjects];
    UIAlertController *alertVC = [UIAlertController alertControllerWithTitle:@"" message:@"Search" preferredStyle:UIAlertControllerStyleAlert];
    [alertVC addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.placeholder = @"StopTime(10~60s)";
        textField.keyboardType = UIKeyboardTypeNumberPad;
        textField.text = [self->system getStopTime];
    }];
    [alertVC addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        UITextField *textField = alertVC.textFields.firstObject;
        NSTimeInterval time = [textField.text intValue];
        [self->system setStopTime:[NSString stringWithFormat:@"%d", (int)time]];
        if (self->asP52N.isConnected) {
            self->searchStatus = YES;
        }
        [SVProgressHUD showWithStatus:@"Searching..."];
        [self->asP52N setSearchDeviceTimeout:time];
        [self->asP52N startSearchDevice];
    }]];
    [alertVC addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {

    }]];
    [self presentViewController:alertVC animated:YES completion:nil];
}
- (IBAction)onClickConnectButton:(id)sender {
    NSLog(@"Connect");
    [self.view endEditing:YES];
    [self->system setIp:self.ipTextField.text];
    [self->system setPort:self.portTextField.text];
    [SVProgressHUD showWithStatus:@"Connecting..."];
    [asP52N connectServerWithIP:[system getIp] remoteport:[[system getPort] intValue]];
}
- (IBAction)onClickDisconnectButton:(id)sender {
    NSLog(@"Disconnect");
    [self.view endEditing:YES];
    [SVProgressHUD showWithStatus:@"Disconnecting..."];
    [asP52N disconnectServer];
}
- (IBAction)onClickStartInventoryButton:(id)sender {
    BOOL mask = NO;
    if (system.getMaskData.length > 0 && system.getMaskAddress.length > 0) {
        mask = YES;
    } else {
        mask = NO;
    }
    switch ([system getInventoryType]) {
        case 0: {
            NSLog(@"startInventoryWithAntenna1");
            if (mask) {                
                [asP52N readDataWithAntenna1:[system getAnt1Status] antenna2:[system getAnt2Status] antenna3:[system getAnt3Status] antenna4:[system getAnt4Status] antenna5:[system getAnt5Status] antenna6:[system getAnt6Status] antenna7:[system getAnt7Status] antenna8:[system getAnt8Status] isContinueScan:[system getIsContinueScan] isUsedTID:NO isUsedTidLength:NO tidLength:0 isUsedUserData:NO userDataOffset:0 userDataLength:0 isUsedReserved:NO reservedOffset:0 reservedLength:0 isUsedPassword:NO accessPassword:@"" isUsedMonza:NO isUsedRfmicron:NO isUsedEMSensor:NO isUsedEPC:NO epcOffset:0 epcLength:0 isUsedMatch:YES matchMemoryBankType:[system getMaskType].intValue + 1 matchData:[system getMaskData] matchStartAddress:[system getMaskAddress].intValue];
            } else {
                [asP52N startInventoryWithAntenna1:[system getAnt1Status] antenna2:[system getAnt2Status] antenna3:[system getAnt3Status] antenna4:[system getAnt4Status] antenna5:[system getAnt5Status] antenna6:[system getAnt6Status] antenna7:[system getAnt7Status] antenna8:[system getAnt8Status] isContinueScan:[system getIsContinueScan]];
            }
        }
            break;
        case 1: {
            NSLog(@"startInventoryTIDWithAntenna1");
            if (mask) {                
                [asP52N readDataWithAntenna1:[system getAnt1Status] antenna2:[system getAnt2Status] antenna3:[system getAnt3Status] antenna4:[system getAnt4Status] antenna5:[system getAnt5Status] antenna6:[system getAnt6Status] antenna7:[system getAnt7Status] antenna8:[system getAnt8Status] isContinueScan:[system getIsContinueScan] isUsedTID:YES isUsedTidLength:YES tidLength:6 isUsedUserData:NO userDataOffset:0 userDataLength:0 isUsedReserved:NO reservedOffset:0 reservedLength:0 isUsedPassword:NO accessPassword:@"" isUsedMonza:NO isUsedRfmicron:NO isUsedEMSensor:NO isUsedEPC:NO epcOffset:0 epcLength:0 isUsedMatch:YES matchMemoryBankType:[system getMaskType].intValue + 1 matchData:[system getMaskData] matchStartAddress:[system getMaskAddress].intValue];
            } else {
                [asP52N startInventoryTIDWithAntenna1:[system getAnt1Status] antenna2:[system getAnt2Status] antenna3:[system getAnt3Status] antenna4:[system getAnt4Status] antenna5:[system getAnt5Status] antenna6:[system getAnt6Status] antenna7:[system getAnt7Status] antenna8:[system getAnt8Status] isContinueScan:[system getIsContinueScan]];
            }
        }
            break;
        case 2: {
            NSLog(@"startInventoryTIDAndUSER");
            if (mask) {
                [asP52N readDataWithAntenna1:[system getAnt1Status] antenna2:[system getAnt2Status] antenna3:[system getAnt3Status] antenna4:[system getAnt4Status] antenna5:[system getAnt5Status] antenna6:[system getAnt6Status] antenna7:[system getAnt7Status] antenna8:[system getAnt8Status] isContinueScan:[system getIsContinueScan] isUsedTID:YES isUsedTidLength:YES tidLength:6 isUsedUserData:YES userDataOffset:0 userDataLength:2 isUsedReserved:YES reservedOffset:0 reservedLength:0 isUsedPassword:NO accessPassword:@"" isUsedMonza:NO isUsedRfmicron:NO isUsedEMSensor:NO isUsedEPC:NO epcOffset:0 epcLength:0 isUsedMatch:YES matchMemoryBankType:[system getMaskType].intValue + 1 matchData:[system getMaskData] matchStartAddress:[system getMaskAddress].intValue];
            } else {
                [asP52N readDataWithAntenna1:[system getAnt1Status] antenna2:[system getAnt2Status] antenna3:[system getAnt3Status] antenna4:[system getAnt4Status] antenna5:[system getAnt5Status] antenna6:[system getAnt6Status] antenna7:[system getAnt7Status] antenna8:[system getAnt8Status] isContinueScan:[system getIsContinueScan] isUsedTID:YES isUsedTidLength:YES tidLength:6 isUsedUserData:YES userDataOffset:0 userDataLength:2 isUsedReserved:YES reservedOffset:0 reservedLength:0 isUsedPassword:NO accessPassword:@"" isUsedMonza:NO isUsedRfmicron:NO isUsedEMSensor:NO isUsedEPC:NO epcOffset:0 epcLength:0 isUsedMatch:NO matchMemoryBankType:AsP52NMemoryBankTypeReserved matchData:@"" matchStartAddress:0];
            }
        }
        case 3: {
            NSLog(@"startInventoryMoistureWithAntenna1");
            if (mask) {
                [asP52N readDataWithAntenna1:[system getAnt1Status] antenna2:[system getAnt2Status] antenna3:[system getAnt3Status] antenna4:[system getAnt4Status] antenna5:[system getAnt5Status] antenna6:[system getAnt6Status] antenna7:[system getAnt7Status] antenna8:[system getAnt8Status] isContinueScan:[system getIsContinueScan] isUsedTID:NO isUsedTidLength:NO tidLength:0 isUsedUserData:NO userDataOffset:0 userDataLength:0 isUsedReserved:YES reservedOffset:11 reservedLength:1 isUsedPassword:NO accessPassword:@"" isUsedMonza:NO isUsedRfmicron:NO isUsedEMSensor:NO isUsedEPC:NO epcOffset:0 epcLength:0 isUsedMatch:YES matchMemoryBankType:[system getMaskType].intValue + 1 matchData:[system getMaskData] matchStartAddress:[system getMaskAddress].intValue];
            } else {
                [asP52N startInventoryMoistureWithAntenna1:[system getAnt1Status] antenna2:[system getAnt2Status] antenna3:[system getAnt3Status] antenna4:[system getAnt4Status] antenna5:[system getAnt5Status] antenna6:[system getAnt6Status] antenna7:[system getAnt7Status] antenna8:[system getAnt8Status] isContinueScan:[system getIsContinueScan]];
            }
        }
            break;
        case 4: {
            NSLog(@"startInventoryMagnusS3WithAntenna1");
            if (mask) {
                [asP52N readDataWithAntenna1:[system getAnt1Status] antenna2:[system getAnt2Status] antenna3:[system getAnt3Status] antenna4:[system getAnt4Status] antenna5:[system getAnt5Status] antenna6:[system getAnt6Status] antenna7:[system getAnt7Status] antenna8:[system getAnt8Status] isContinueScan:[system getIsContinueScan] isUsedTID:NO isUsedTidLength:NO tidLength:0 isUsedUserData:NO userDataOffset:0 userDataLength:0 isUsedReserved:NO reservedOffset:0 reservedLength:0 isUsedPassword:NO accessPassword:@"" isUsedMonza:NO isUsedRfmicron:YES isUsedEMSensor:NO isUsedEPC:NO epcOffset:0 epcLength:0 isUsedMatch:YES matchMemoryBankType:[system getMaskType].intValue + 1 matchData:[system getMaskData] matchStartAddress:[system getMaskAddress].intValue];
            } else {
                [asP52N startInventoryMagnusS3WithAntenna1:[system getAnt1Status] antenna2:[system getAnt2Status] antenna3:[system getAnt3Status] antenna4:[system getAnt4Status] antenna5:[system getAnt5Status] antenna6:[system getAnt6Status] antenna7:[system getAnt7Status] antenna8:[system getAnt8Status] isContinueScan:[system getIsContinueScan]];
            }
        }
            break;

            break;
        default:
            break;
    }
}
- (IBAction)onClickStopInventoryButton:(id)sender {
    NSLog(@"stopReadData");
    [asP52N stopReadData];
}
- (IBAction)onClickClearButton:(id)sender {
    [dataArray removeAllObjects];
    [dataDic removeAllObjects];
    count = 0;
    total = 0;
    self.countLabel.text = @"0";
    self.totalLabel.text = @"0";
    [_tableView reloadData];
}
- (IBAction)onClickMore:(UIBarButtonItem *)sender {
    if (!inventoryStatus) {
        [self performSegueWithIdentifier:@"moreID" sender:nil];
    }
}
#pragma mark - Table view data source
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return dataArray.count;
}
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    if ([system getInventoryType] == InventoryTypeEPC) {
        NSString *cellIdentifier = @"EPCDataCell";
        EPCDataCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
        if(!cell) {
            cell = [[[NSBundle mainBundle] loadNibNamed:@"EPCDataCell" owner:self options:nil] firstObject];
        }
        EPCData * epcData = [dataArray objectAtIndex:indexPath.row];
        cell.epcLabel.text = [NSString stringWithFormat:@"EPC:%@", epcData.epc];
        cell.phaseLabel.text = [NSString stringWithFormat:@"PC:%@  ANT:%@  Phase:%@", epcData.pc, epcData.antenna, epcData.phase];
        cell.countLabel.text = [NSString stringWithFormat:@"Channel:%@  RSSI:%@  Count:%d", epcData.channel,epcData.rssi,epcData.count];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.accessoryType = UITableViewCellAccessoryNone;
        return cell;
    } else {
        NSString *cellIdentifier = @"TIDDataCell";
        TIDDataCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
        if(!cell) {
            cell = [[[NSBundle mainBundle] loadNibNamed:@"TIDDataCell" owner:self options:nil] firstObject];
        }
        EPCData * epcData = [dataArray objectAtIndex:indexPath.row];
        cell.epcLabel.text = [NSString stringWithFormat:@"EPC:%@", epcData.epc];
        cell.phaseLabel.text = [NSString stringWithFormat:@"PC:%@  ANT:%@  Phase:%@", epcData.pc, epcData.antenna, epcData.phase];
        cell.countLabel.text = [NSString stringWithFormat:@"Channel:%@  RSSI:%@  Count:%d", epcData.channel,epcData.rssi,epcData.count];
        if ([system getInventoryType] == InventoryTypeTID) {
            cell.tidLabel.text = [NSString stringWithFormat:@"TID:%@", epcData.tid];
        } else if ([system getInventoryType] == InventoryTypeMoisture) {
            cell.tidLabel.text = [NSString stringWithFormat:@"Moisture:%@", epcData.moisture];
        } else if ([system getInventoryType] == InventoryTypeMagnusS3) {
            cell.tidLabel.text = [NSString stringWithFormat:@"MagnusS3:%@", epcData.temperature];
        } else if ([system getInventoryType] == InventoryTypeUSER) {
            cell.tidLabel.text = [NSString stringWithFormat:@"USER:%@", epcData.userData];
        }
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.accessoryType = UITableViewCellAccessoryNone;
        return cell;
    }
}
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    _epcData = [dataArray objectAtIndex:indexPath.row];
    [self performSegueWithIdentifier:@"DataListID" sender:nil];
}
- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath{
    if ([cell respondsToSelector:@selector(setLayoutMargins:)]) {
        [cell setLayoutMargins:UIEdgeInsetsZero];
    }
    if ([cell respondsToSelector:@selector(setSeparatorInset:)]){
        [cell setSeparatorInset:UIEdgeInsetsZero];
    }
}
#pragma mark - AsP52NDelegate
- (void)AsP52N:(AsP52N *)AsP52N searchingIP:(NSString *)IP isFinish:(BOOL)isFinish{
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if(isFinish == YES)
        {
            if([SVProgressHUD isVisible]){
                [SVProgressHUD dismiss];
            }
            if ([self->arrIPs count]>0) {
                [self showAlertViewIPs];
            } else {
                UIAlertController * alert = [UIAlertController alertControllerWithTitle:@"Search" message:@"Not found Devices" preferredStyle:UIAlertControllerStyleAlert];
                [alert addAction:[[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil] init]];
                [self presentViewController:alert animated:YES completion:nil];
            }
        } else {
            if (![self->arrIPs containsObject:IP]) {
                [self->arrIPs addObject:IP];
            }
        }
    }];
}
- (void)AsP52N:(AsP52N *)AsP52N changeNetworkState:(AsP52NetworkState)state error:(NSError *)error{
    NSLog(@"AsP52NetworkState = %lu error: %@",(unsigned long)state,error);
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        if (state == AsP52NetworkStateConnected) {
            [self->asP52N stopInventory];
            self.settingBarItem.enabled = YES;
            self.connectButton.userInteractionEnabled = NO;
            self.connectButton.backgroundColor = [UIColor lightGrayColor];
            self.disConnectButton.userInteractionEnabled = YES;
            self.disConnectButton.backgroundColor = [UIColor redColor];
            self.startButton.userInteractionEnabled = YES;
            self.startButton.backgroundColor = [UIColor redColor];
            self.stopButton.userInteractionEnabled = NO;
            self.stopButton.backgroundColor = [UIColor lightGrayColor];
            self.clearButton.userInteractionEnabled = YES;
            self.clearButton.backgroundColor = [UIColor redColor];
            self.tableView.userInteractionEnabled = YES;
            self.temperatureLabel.text = @"";
            [self.tableView reloadData];
            [self getTemperature];
        } else {
            self.settingBarItem.enabled = NO;
            self.connectButton.userInteractionEnabled = YES;
            self.connectButton.backgroundColor = [UIColor redColor];
            self.disConnectButton.userInteractionEnabled = NO;
            self.disConnectButton.backgroundColor = [UIColor lightGrayColor];
            self.startButton.userInteractionEnabled = NO;
            self.startButton.backgroundColor = [UIColor lightGrayColor];
            self.stopButton.userInteractionEnabled = NO;
            self.stopButton.backgroundColor = [UIColor lightGrayColor];
            self.clearButton.userInteractionEnabled = NO;
            self.clearButton.backgroundColor = [UIColor lightGrayColor];
            self.tableView.userInteractionEnabled = NO;
            self.temperatureLabel.text = @"";
        }
        if (error) {
            UIAlertController * alert = [UIAlertController alertControllerWithTitle:@"Error" message:[NSString stringWithFormat:@"%@", error.localizedFailureReason] preferredStyle:UIAlertControllerStyleAlert];
            [alert  addAction:[[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil] init]];
            [self presentViewController:alert animated:YES completion:nil];
        }
        if (self->searchStatus == YES) {
            self->searchStatus = NO;
            return;;
        }
        if ([SVProgressHUD isVisible]){
            [SVProgressHUD dismissWithDelay:0.2];
        }
    }];
}
- (void)AsP52N:(AsP52N *)AsP52N didStopInventoryExecutionStatus:(AsP52NExecutionStatus)statusCode{
    NSLog (@"didStopInventoryExecutionStatus = %lu",(unsigned long)statusCode);
    self.stopButton.backgroundColor = [UIColor lightGrayColor];
    self.stopButton.userInteractionEnabled = NO;
    self.startButton.backgroundColor = [UIColor redColor];
    self.startButton.userInteractionEnabled = YES;
    inventoryStatus = NO;
}
- (void)AsP52N:(AsP52N *)AsP52N didStartInventoryExecutionStatus:(AsP52NExecutionStatus)statusCode{
    NSLog (@"didStartInventoryExecutionStatus = %lu",(unsigned long)statusCode);
    self.stopButton.backgroundColor = [UIColor redColor];
    self.stopButton.userInteractionEnabled = YES;
    self.startButton.backgroundColor = [UIColor lightGrayColor];
    self.startButton.userInteractionEnabled = NO;
    inventoryStatus = YES;
}
- (void)AsP52N:(AsP52N *)AsP52N didInventoryEndExecutionStatus:(AsP52NExecutionStatus)statusCode;{
    NSLog (@"didInventoryEndExecutionStatus = %lu",(unsigned long)statusCode);
    self.stopButton.backgroundColor = [UIColor lightGrayColor];
    self.stopButton.userInteractionEnabled = NO;
    self.startButton.backgroundColor = [UIColor redColor];
    self.startButton.userInteractionEnabled = YES;
    inventoryStatus = NO;
}
- (void)AsP52N:(AsP52N *)AsP52N receivedInventoryCustomDataWithRSSI:(NSString *)RSSI channel:(NSString *)channel phase:(NSString *)phase antenna:(NSString *)antenna PC:(NSString *)PC EPC:(NSString *)EPC TID:(NSString *)TID userData:(NSString *)userData reserved:(NSString *)reserved epcBank:(NSString *)epcBank moisture:(NSString *)moisture temperature:(NSString *)temperature{
    NSLog (@"APP epcData = %@, pcData = %@, ANT = %@, phase = %@,channel = %@, RSSI = %@, userData = %@, reserved = %@, epcBank = %@, moisture = %@, temperature = %@, TID = %@",EPC,PC,antenna,phase,channel,RSSI,userData,reserved,epcBank,moisture,temperature,TID);
    total ++;
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        EPCData * epcData = [self->dataDic objectForKey:EPC];
        if (epcData) {
            epcData.count = epcData.count + 1;
            epcData.antenna = antenna;
            epcData.phase = phase;
            epcData.channel = channel;
            epcData.rssi = RSSI;
            epcData.tid = TID;
            epcData.moisture = moisture;
            epcData.temperature = [NSString stringWithFormat:@"%.2f °C", [temperature floatValue] * 0.01];
            epcData.epcBank = epcBank;
            epcData.userData = userData;
            epcData.reserved = reserved;
        } else {
            epcData= [[EPCData alloc] init];
            epcData.antenna = antenna;
            epcData.phase = phase;
            epcData.channel = channel;
            epcData.rssi = RSSI;
            epcData.pc = PC;
            epcData.epc = EPC;
            epcData.tid = TID;
            epcData.moisture = moisture;
            epcData.temperature = [NSString stringWithFormat:@"%.2f °C", [temperature floatValue] / 100];
            epcData.epcBank = epcBank;
            epcData.userData = userData;
            epcData.reserved = reserved;
            epcData.count = 1;
            [self->dataArray addObject:epcData];
            [self->dataDic setObject:epcData forKey:EPC];
            self->count ++;
        }
        [[NSOperationQueue mainQueue] addOperationWithBlock:^{
            self.countLabel.text = [NSString stringWithFormat:@"%d", self->count];
            self.totalLabel.text = [NSString stringWithFormat:@"%d", self->total];
            [self->_tableView reloadData];
        }];
    }];
}
- (void)AsP52N:(AsP52N *)AsP52N receivedInventoryStatus:(AsP52NExecutionStatus)statusCode {
    NSString *failedString = [asP52N getMessageWithResultCode:statusCode];
    [Utils showAlert:@"Fail" message:failedString];
}
- (void)AsP52N:(AsP52N *)AsP52N receivedUnknowCommand:(NSData *)data{
    NSLog(@"receivedUnknowCommand data = %@", data);
}
- (void)AsP52N:(AsP52N *)AsP52N receivedUnknowError:(NSError *)error {
    NSLog(@"receivedUnknowError error = %@", error);
}
- (void)AsP52N:(AsP52N *)AsP52N receivedAlarmTemperature:(NSString *)alarmTemperature currentTemperature:(NSString *)currentTemperature{
    NSLog(@"receivedAlarmTemperature alarmTemperature = %@ currentTemperature = %@", alarmTemperature,currentTemperature);
    if ([currentTemperature intValue] < [alarmTemperature intValue]){
        UIAlertController * alert = [UIAlertController alertControllerWithTitle:@"Warning" message:@"The reader is too hot. Inventory will be stopped to avoid overheating." preferredStyle:UIAlertControllerStyleAlert];
        [alert addAction:[[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil] init]];
        [self presentViewController:alert animated:YES completion:nil];
    }
}
- (void)AsP52N:(AsP52N *)AsP52N receivedGetTemperatureAlarmTemperature:(NSString *)alarmTemperature currentTemperature:(NSString *)currentTemperature{
    NSLog(@"receivedGetTemperatureAlarmTemperature alarmTemperature = %@ currentTemperature = %@", alarmTemperature,currentTemperature);
    if ([currentTemperature intValue] < [alarmTemperature intValue]){
        self.temperatureLabel.textColor = [UIColor greenColor];
    } else {
        self.temperatureLabel.textColor = [UIColor redColor];
    }
    self.temperatureLabel.text = [NSString stringWithFormat:@"%@°C",currentTemperature];
}
#pragma mark - Private
- (void)showAlertViewIPs
{
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Found Device"
                                                                   message:@"Please select device IP"
                                                            preferredStyle:UIAlertControllerStyleActionSheet];
    for (NSString *ipString in arrIPs) {
        UIAlertAction *ip = [UIAlertAction actionWithTitle:ipString
                                                     style:UIAlertActionStyleDefault
                                                   handler:^(UIAlertAction * action) {
            self.ipTextField.text = ipString;
            [self->system setIp:self.ipTextField.text];
            [SVProgressHUD showWithStatus:@"Connecting..."];
            [self->asP52N connectServerWithIP:self.ipTextField.text remoteport:[[self->system getPort] intValue]];
        }];
        [alert addAction:ip];
    }
    
    UIAlertAction *cancel = [UIAlertAction actionWithTitle:@"Cancel"
                                                     style:UIAlertActionStyleDefault
                                                   handler:nil];
    [alert addAction:cancel];
    
    UIBarButtonItem *button;
    alert.popoverPresentationController.barButtonItem = button;
    alert.popoverPresentationController.sourceView = self.view;
    [self presentViewController:alert animated:YES completion:nil];
}
- (void)restartDevice {
    [self setDisconnectUI];
}
- (void)factoryRest {
    [self setDisconnectUI];
}
- (void)setDisconnectUI {
    self.settingBarItem.enabled = NO;
    self.connectButton.userInteractionEnabled = YES;
    self.connectButton.backgroundColor = [UIColor redColor];
    self.disConnectButton.userInteractionEnabled = NO;
    self.disConnectButton.backgroundColor = [UIColor lightGrayColor];
    self.startButton.userInteractionEnabled = NO;
    self.startButton.backgroundColor = [UIColor lightGrayColor];
    self.stopButton.userInteractionEnabled = NO;
    self.stopButton.backgroundColor = [UIColor lightGrayColor];
    self.clearButton.userInteractionEnabled = NO;
    self.clearButton.backgroundColor = [UIColor lightGrayColor];
    self.tableView.userInteractionEnabled = NO;
}
#pragma mark - System
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if ([segue.identifier isEqualToString:@"DataListID"]) {
        DataListController *dataVC = segue.destinationViewController;
        dataVC.epcData = _epcData;
    }
}
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
}
@end
